import pygame, sys, os
import data
import fontmanager

import building, env, gameobjects, utils

from pygame.locals import *

class DashManDX:
    def __init__(self):
        #pygames stuff
        pygame.init()
        self.SIZE = SCREENW, SCREENH = 640, 480
        self.screen = pygame.display.set_mode(self.SIZE)
        pygame.display.set_caption('Dashman DX')
        pygame.mouse.set_visible(1)
        self.clock = pygame.time.Clock()

        self.mode = 0 # 0 = mainmenu

        self.mainmenu = MainMenu(self)

    def show_main_menu(self):
        self.mode = 0
        self.mainmenu.mainloop()

    def start_main_game(self):
        self.mode = 1
        #game = Game()
        for l in range(1,8):
            if self.mode == 1:            
                self.play_level("level%d"%l)
        self.mode = 0

    def play_level(self, lvlname):
        game = Game()
        path = os.path.join(os.path.join("lvls","normal"),lvlname)
        game.play_level(path, self)


    def play_custom_level(self, lvlname):
        game = Game()
        path = os.path.join(os.path.join("lvls","custom"),lvlname)
        game.play_level(path, self)

    def show_level_editor_menu(self):
        menu = CustomMapMenu(self, 1)


    def play_custom_levels(self):
        menu = CustomMapMenu(self)

    def start_level_editor(self, lvl=""):
        self.mode = 2
        editor = LevelEditor(self, lvl)
        editor.mainloop()

    def quit_game(self):
        sys.exit(0)


class MainMenu:
    class MenuOption:
        def __init__(self, name, function):
            self.name = name
            self.function = function
    
    def __init__(self, mainapp):
        self.mainapp = mainapp
        self.fontmgr = fontmanager.mFontManager((('44v2.ttf', 24),('44v2.ttf', 36)))
        self.background = data.load_image("bg.jpg")[0].convert()

        self.menu = []
        self.menu.append(self.MenuOption("Start Game", DashManDX.start_main_game))
        self.menu.append(self.MenuOption("Play custom level", DashManDX.play_custom_levels))
        self.menu.append(self.MenuOption("Level editor", DashManDX.show_level_editor_menu))
        self.menu.append(self.MenuOption("Quit Game", DashManDX.quit_game))
                
        self.choice = 0
        
    def mainloop(self):
        while self.mainapp.mode == 0:
            self.mainapp.clock.tick(60)
            self.handle_input()

            self.draw(self.mainapp.screen)
            
    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE: sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    pass
                
                elif event.key == pygame.K_UP:
                    self.choice -= 1
                    if self.choice < 0:
                        self.choice = 3
                elif event.key == pygame.K_DOWN:
                    self.choice += 1
                    if self.choice >= 4:
                        self.choice = 0                        
                elif event.key == pygame.K_LEFT:
                    pass
                elif event.key == pygame.K_RIGHT or event.key == pygame.K_RETURN:
                    self.menu[self.choice].function(self.mainapp)


    def draw(self, surface):
        surface.blit(self.background, (0,0))

        x, y = (150,150)
        numitem = 0
        
        for option in self.menu:
            if numitem == self.choice:
                size = 36
                x_off = -20
            else:
                size = 24
                x_off = 0
            self.fontmgr.Draw(surface, '44v2.ttf', size, option.name,
                              (x+x_off, y), (150,42,42), 'left', 'center', True)
            y += size+15
            numitem += 1
        
        pygame.display.flip()

class CustomMapMenu:
    def __init__(self, mainapp, edit=0):
        self.mainapp = mainapp
        self.edit = edit
        self.fontmgr = fontmanager.mFontManager((('44v2.ttf', 30),('44v2.ttf', 24),('44v2.ttf', 18)))
        self.background = data.load_image("bg.jpg")[0].convert()

        self.load_menu()
                
        self.choice = 0
        self.mainapp.mode = 3
        self.mainloop()


    def load_menu(self):
        if self.edit:
            self.menu = ["-[ NEW LEVEL ]-"]+data.get_custom_levels()
        else:
            self.menu = data.get_custom_levels()
        
    def mainloop(self):
        while self.mainapp.mode == 3:
            self.mainapp.clock.tick(60)
            self.handle_input()
            self.draw(self.mainapp.screen)
            
    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE: self.mainapp.mode = 0
                elif event.key == pygame.K_SPACE:
                    pass
                
                elif event.key == pygame.K_UP:
                    self.choice -= 1
                    if self.choice < 0:
                        self.choice = len(self.menu)-1
                elif event.key == pygame.K_DOWN:
                    self.choice += 1
                    if self.choice >= len(self.menu):
                        self.choice = 0                        
                elif event.key == pygame.K_LEFT:
                    pass
                elif event.key == pygame.K_RIGHT or event.key == pygame.K_RETURN:
                    if not self.edit:
                        self.mainapp.mode = 1
                        self.mainapp.play_custom_level(self.menu[self.choice])
                        self.mainapp.mode = 3
                    else:
                        if self.choice == 0:
                            self.mainapp.start_level_editor()
                        else:
                            self.mainapp.start_level_editor(self.menu[self.choice])
                        self.load_menu()
                        self.mainapp.mode = 3


    def draw(self, surface):
        surface.blit(self.background, (0,0))

        self.fontmgr.Draw(surface, '44v2.ttf', 24, "Select a level",
                              pygame.Rect(0,0,640,80), (150,42,42), 'center', 'center', True)


        if len(self.menu) < 10:
            n = 0
            for i in self.menu:
                if n == self.choice:
                    size = 30
                else:
                    
                    size = 24
                try:
                    self.fontmgr.Draw(surface, '44v2.ttf', size, i,
                                      pygame.Rect(0,100+n*30,640,20), (150,42,42), 'left', 'center', True)
                except IndexError:
                    pass
                n += 1

        elif (self.choice > (len(self.menu)-10)):
            #end of list
            for offset in range(10):
                if (len(self.menu)-10+offset) == self.choice:
                    size = 30
                else:
                    
                    size = 24
                try:
                    self.fontmgr.Draw(surface, '44v2.ttf', size, self.menu[len(self.menu)-10+offset],
                                      pygame.Rect(0,100+offset*30,640,20), (150,42,42), 'left', 'center', True)
                except IndexError:
                    pass
        else:
            #not at end of list
            for offset in range(10):
                if offset == 0:
                    size = 30
                else:
                    size = 24
                try:
                    self.fontmgr.Draw(surface, '44v2.ttf', size, self.menu[self.choice+offset],
                                      pygame.Rect(0,100+offset*30,640,20), (150,42,42), 'left', 'center', True)
                except IndexError:
                    pass

        pygame.display.flip()



class Game:
    def __init__(self):
        pass

    def play_level(self, lvlname, mainapp):
        self.mainapp = mainapp
        self.background = data.load_image("bg.jpg")[0].convert()
        self.building = building.Building(lvlname)
        self.fontmgr = fontmanager.mFontManager((('44v2.ttf', 24),('44v2.ttf', 36),('44v2.ttf', 40)))

        herox, heroy = self.building.start
        self.hero = gameobjects.Hero(herox, heroy, self.building, mainapp.screen)

        self.allsprites = pygame.sprite.RenderPlain((self.hero,))

        self.building.reset_timer()

        self.level_start()
        self.mainloop()
        
        
        
    def mainloop(self):
        while self.mainapp.mode == 1:
            self.mainapp.clock.tick(60)
            self.handle_input()
            self.allsprites.update()
            self.building.tick_timer()
            self.draw(self.mainapp.screen)

            if self.building.won:
                self.level_complete()
                break

    def level_start(self):
        #stuff to do at start of level
        #display things and so on

        tickstoshow = 60*1.5 #1.5 seconds
        while tickstoshow:
            self.mainapp.clock.tick(60)
            self.allsprites.update()
            self.draw(self.mainapp.screen, self.building.name, "start!")
            tickstoshow -= 1



    def level_complete(self):
        tickstoshow = 60*3 #3 seconds
        while tickstoshow:
            self.mainapp.clock.tick(60)
            #self.allsprites.update()
            
            self.draw(self.mainapp.screen, "level complete","%.2f seconds" % self.building.get_timer())
            tickstoshow -= 1

        

    def handle_input(self):
        #key events
        busy = 0
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.mainapp.mode = 0 #sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    if not busy:
                        self.hero.dash()
                    busy = 1
                elif event.key == pygame.K_UP:
                    if not busy:
                        self.hero.moveUp()
                    busy = 1
                elif event.key == pygame.K_LEFT:
                    self.hero.moveLeft()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveRight()

            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    self.hero.moveLeft(1)
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveRight(1)

    def draw(self, surface, text1="", text2=""):
        surface.blit(self.background, (0, 0))

        #grid.draw(surface)
        self.building.draw(surface)
        self.allsprites.draw(surface)

        if text1:
            self.fontmgr.Draw(self.mainapp.screen, '44v2.ttf', 36, text1,
                              pygame.Rect(0,0,640,240), (150,42,42), 'center', 'center', True)
        if text2:
            self.fontmgr.Draw(self.mainapp.screen, '44v2.ttf', 36, text2,
                              pygame.Rect(0,120,640,240), (150,42,42), 'center', 'center', True)
        
        pygame.display.flip()

class NameMenu:
    def __init__(self, surface):
        self.surface = surface
        self.fontmgr = fontmanager.mFontManager((('44v2.ttf', 22),('44v2.ttf', 16)))
        self.background = data.load_image("bg.jpg")[0].convert()
        self.mapname = []
        self.running = 1
        self.mainloop()
        
    def mainloop(self):
        while self.running:
            self.draw(self.surface)
            self.handle_input()

    def draw(self, surface):
        surface.blit(self.background, (0, 0))

        self.fontmgr.Draw(surface, '44v2.ttf', 22, "Name your map and press return",
                              pygame.Rect(0,0,640,240), (150,42,42), 'center', 'center', True)
        self.fontmgr.Draw(surface, '44v2.ttf', 16, "The name must be at least 3 characters",
                             pygame.Rect(0,40,640,240), (150,42,42), 'center', 'center', True)

        self.fontmgr.Draw(surface, '44v2.ttf', 22, "".join(self.mapname),
                          pygame.Rect(0,120,640,240), (150,42,42), 'center', 'center', True)
        
        pygame.display.flip()

    def handle_input(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_RETURN: #save
                    if len(self.mapname) >= 3:
                        self.running = 0
                elif event.key == pygame.K_BACKSPACE:
                    if self.mapname:
                        self.mapname.pop()
                if event.key in range(256):
                    key = chr(event.key)
                    if key.isalnum():
                        self.mapname.append(key)

    
        
                    

class LevelEditor:
    def __init__(self, mainapp, lvl=""):
        self.mainapp = mainapp
        self.background = data.load_image("bg.jpg")[0].convert()
        if lvl:
            self.building = building.Building("lvls/custom/%s"%lvl,editing=1)
        else:
            self.building = building.Building(editing=1)
        self.fontmgr = fontmanager.mFontManager((('44v2.ttf', 24),('44v2.ttf', 36)))

        herox, heroy = self.building.start
        self.hero = gameobjects.Hero(herox, heroy, self.building, mainapp.screen)

        self.allsprites = pygame.sprite.RenderPlain((self.hero,))

        self.building.reset_timer()


        self.grid = utils.Grid()
        self.to_make = 3
        self.last_point = (-1,-1)

        if not lvl:
            self.get_name()
        
        #self.level_start()
        self.mainloop()



    def save_map(self):
        self.building.save_data()
        msg = utils.show_message("%s has been saved" % self.building.name, self.mainapp.screen)


    def get_name(self):
        menu = NameMenu(self.mainapp.screen)
        self.building.name = "".join(menu.mapname)
    

    def respawn_hero(self):
        herox, heroy = self.building.start
        self.hero.x = herox
        self.hero.y = heroy
        self.hero.v_x = self.hero.v_y = 0

    def mainloop(self):
        while self.mainapp.mode == 2:
            self.mainapp.clock.tick(60)
            self.handle_input()
            self.allsprites.update()
            self.building.tick_timer()
            self.draw(self.mainapp.screen)

            if self.building.won:
                self.respawn_hero()
                self.building.won = 0

    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    print "test"
                    self.mainapp.mode = 0 #sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    self.hero.dash()
                elif event.key == pygame.K_UP:
                    self.hero.moveUp()
                elif event.key == pygame.K_LEFT:
                    self.hero.moveLeft()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveRight()
                elif event.key == pygame.K_DOWN:
                    self.respawn_hero()
                    

                #LEVEL EDITOR EVENTS
                elif event.key == pygame.K_0: #none
                    self.to_make = 0
                elif event.key == pygame.K_1: #start
                    self.to_make = 1
                elif event.key == pygame.K_2: #end
                    self.to_make = 2
                elif event.key == pygame.K_3: #floor / wall
                    self.to_make = 3
                elif event.key == pygame.K_4: #trampoline
                    self.to_make = 4

                elif event.key == pygame.K_s: #save
                    filename = self.save_map()
                elif event.key == pygame.K_f: #fix level
                    print "fix level!"
                    self.building.fix_level()
                    
            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    self.hero.moveStop()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveStop()

            #MOUSE EVENTS FOR KEY EDITOR!!!
            elif event.type == MOUSEBUTTONDOWN:

                (lb, mb, rb)= pygame.mouse.get_pressed()
                x = (pygame.mouse.get_pos()[0]+8)/16
                y = 20-(pygame.mouse.get_pos()[1]+12)/24
                
                print "clicked:",x,y

                #kill what is right clicked!
                if rb:
                    self.building.kill_at(x,y)

                #CREATION!!!
                if lb:
                    if self.to_make == 1: #start
                        print "setting start!",x,"x",y
                        self.building.set_start(x,y)
                    elif self.to_make == 2: #end
                        print "setting end!",x,"x",y
                        self.building.set_end(x,y)
                    elif self.to_make == 3: #floor / wall
                        #check if any last point
                        if self.last_point == (-1,-1): # no last point
                            self.last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a wall!
                            if (self.last_point[0] == x ) and (self.last_point[1] != y):
                                if self.last_point[1] < y:
                                    w_y = self.last_point[1]
                                    height = y - self.last_point[1]
                                else:
                                    w_y = y
                                    height = self.last_point[1] -y
                                self.building.make_wall(env.Wall(x, w_y, height))

                            #Draw a floor!
                            elif (self.last_point[1] == y) and (self.last_point[0] != x):
                                if self.last_point[0] < x:
                                    x1 = self.last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = self.last_point[0]
                                self.building.make_floor(env.Floor(y, x1, x2))

                            #Draw a floor and a wall!
                            else:
                                lastx, lasty = self.last_point
                                
                                if lastx < x:
                                    x1 = lastx
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = lastx
                                    
                                if lasty < y:
                                    w_y = lasty
                                    height = y - lasty
                                else:
                                    w_y = y
                                    height = lasty - y

                                self.building.make_floor(env.Floor(lasty, x1, x2))
                                self.building.make_wall(env.Wall(x, w_y, height))
                                
                            self.last_point = (-1, -1)
                    elif self.to_make == 4: #trampoline
                        if self.last_point == (-1,-1): # no last point
                            self.last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a trampoline floor!
                            if (self.last_point[1] == y) and (self.last_point[0] != x):
                                if self.last_point[0] < x:
                                    x1 = self.last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = self.last_point[0]
                                self.building.make_floor(env.Trampoline(y, x1, x2))
                            self.last_point = (-1, -1)
                        






    def draw(self, surface):
        surface.blit(self.background, (0, 0))

        self.grid.draw(surface)
        self.building.draw(surface)
        self.allsprites.draw(surface)

        
        pygame.display.flip()



def main():
    mainapp = DashManDX()
    
    mainapp.show_main_menu()
    #mainapp.play_level("lvl9")
    



def do_later():

    
    #my stuff
    building = Building("lvl9")

   
    
    herox, heroy = building.start
    hero = Hero(herox, heroy, building, screen)

    allsprites = pygame.sprite.RenderPlain((hero,))

    grid = Grid()
    to_make = 0
    last_point = (-1,-1)

    building.reset_timer()
    
    while(1):
        clock.tick(60)
        #hero.update()
        allsprites.update()
        building.tick_timer()

        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
		if event.key == pygame.K_ESCAPE: sys.exit(0)
		elif event.key == pygame.K_SPACE:
                    hero.dash()
                
		elif event.key == pygame.K_UP:
                    hero.moveUp()
		elif event.key == pygame.K_LEFT:
                    hero.moveLeft()
		elif event.key == pygame.K_RIGHT:
                    hero.moveRight()
                elif event.key == pygame.K_s:
                    building.save_data()

                elif event.key == pygame.K_0: #none
                    to_make = 1
                elif event.key == pygame.K_1: #start
                    to_make = 1
                elif event.key == pygame.K_2: #end
                    to_make = 2
                elif event.key == pygame.K_3: #floor / wall
                    to_make = 3
                elif event.key == pygame.K_4: #wall
                    to_make = 4
                elif event.key == pygame.K_5: #trampoline
                    to_make = 5
                elif event.key == pygame.K_6:
                    to_make = 6
                elif event.key == pygame.K_7:
                    to_make = 7
                    

            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_DOWN: pass
		elif event.key == pygame.K_UP:
                    pass
		elif event.key == pygame.K_LEFT:
                    hero.moveStop()
		elif event.key == pygame.K_RIGHT:
                    hero.moveStop()
            
            elif event.type == MOUSEBUTTONDOWN:

                (lb, mb, rb)= pygame.mouse.get_pressed()
                x = (pygame.mouse.get_pos()[0]+8)/16
                y = 20-(pygame.mouse.get_pos()[1]+12)/24

                #print pygame.mouse.get_pos()
                print x,y


                if rb:
                    building.kill_at(x,y)

                if lb:
                    if to_make == 1: #start
                        print "setting start!",x,"x",y
                        building.set_start(x,y)
                    elif to_make == 2: #end
                        print "setting end!",x,"x",y
                        building.set_end(x,y)
                    elif to_make == 3: #floor
                        #check if any last point
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a wall!
                            if (last_point[0] == x ) and (last_point[1] != y):
                                if last_point[1] < y:
                                    w_y = last_point[1]
                                    height = y - last_point[1]
                                else:
                                    w_y = y
                                    height = last_point[1] -y
                                building.make_wall(Wall(x, w_y, height))

                            #Draw a floor!
                            elif (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Floor(y, x1, x2))
                            last_point = (-1, -1)
                    elif to_make == 4: #trampoline
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a trampoline floor!
                            if (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Trampoline(y, x1, x2))
                            last_point = (-1, -1)
                        


                
                    

                
        screen.blit(background, (0, 0))

        #grid.draw(screen)
        building.draw(screen)
        allsprites.draw(screen)

        pygame.display.flip()
